#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 pcd_creator.py

 @author East Japan Accounting Center Co., Ltd

 (c)2023 The University of Aizu
 This software is released under the New BSD License.
"""

from datetime import datetime, timedelta, timezone
import logging
import os

import numpy as np

from point_cloud_field import PointCloudField

logger = logging.getLogger(__name__)

JST = timezone(timedelta(hours=+9), 'JST')
ROBOT_RELATIVE_ROOT_DIR = "db"
PCD_VERSION = 0.7


class CreateDate:
    @staticmethod
    def convert_rel_2_abs(relative_file_path):
        """

        Parameters
        ----------
        relative_file_path

        Returns
        -------

        """
        return os.path.join(os.environ["RDR_ROOT"], relative_file_path)

    @staticmethod
    def convert_abs_2_rel(absolute_file_path):
        """

        Parameters
        ----------
        absolute_file_path

        Returns
        -------

        """
        return absolute_file_path.replace(os.environ["RDR_ROOT"], "")

    @classmethod
    def make_file_path(cls, robot_id, file_category, basename_without_ext, extension):
        """
        make file path

        Param
        ----------
        robot_id: str
            Robot types
        file_category: str
            File types
        extension: str
            File extension

        Return
        ----------
        file_path: str
            path information including map file name
        """
        file_path = None
        if robot_id is None:
            return file_path
        cur_time = datetime.now(JST)
        base_dir = os.path.join(ROBOT_RELATIVE_ROOT_DIR, robot_id, file_category)
        time_dir = "{year}{month:02}{day:02}".format(year=cur_time.year, month=cur_time.month, day=cur_time.day)

        # Define the storage destination directory and generate the directory
        target_dir = os.path.join(base_dir, time_dir)
        abs_dir = cls.convert_rel_2_abs(target_dir)
        if not os.path.isdir(abs_dir):
            os.makedirs(abs_dir)

        file_name = "{}.{}".format(basename_without_ext, extension)
        file_path = os.path.join(target_dir, file_name)
        logger.info("pcd_creator: %s", file_path)

        return file_path


class PcdCreator:
    def __init__(self):
        pass

    def run(self, doc):

        result = False
        path = ""

        basename_without_ext = os.path.splitext(os.path.basename(doc["data_path"]))[0]
        pcd_file_path = CreateDate.make_file_path(doc["id"], "pcd", basename_without_ext, "pcd")
        if pcd_file_path is None:
            return result, path

        pcd_abs_file_path = CreateDate.convert_rel_2_abs(pcd_file_path)
        if self.create_pcd(doc, pcd_abs_file_path):
            result = True
            path = pcd_abs_file_path

        return result, path

    @staticmethod
    def _analysis_fields(fields: list):

        fields_dict_list = {field["name"]: PointCloudField(field) for field in fields}
        return fields_dict_list

    @staticmethod
    def _read_binary(file_path):
        byte_data = None
        if os.path.isfile(file_path):
            with open(file_path, "rb") as byte_file:
                byte_data = byte_file.read()

                return byte_data
        else:
            return byte_data

    def create_pcd(self, point_dict, file_path):

        result = False

        binary_file_path = os.environ["RDR_ROOT"] + point_dict["data_path"]
        bin_a = self._read_binary(binary_file_path)
        point_a = [int(byte_data) for byte_data in bin_a]
        point_a = np.array(point_a)
        point2_a = point_a.reshape(-1, point_dict["point_step"])

        width = point_dict["width"]
        height = point_dict["height"]
        offset_dict = self._analysis_fields(point_dict["fields"])
        big_endian = point_dict["is_bigendian"]
        points = int(width * height)

        try:
            with open(file_path, "w") as pcd_file:
                # set header
                header_fields = ""
                header_counts = ""
                header_size = ""
                header_type = ""
                offset_key_list = list(offset_dict.keys())
                for key_name in offset_key_list:
                    header_fields += " {}".format(key_name)
                    header_counts += " {}".format(offset_dict[key_name].get_count())
                    header_size += " {}".format(offset_dict[key_name].get_size())
                    header_type += " {}".format(offset_dict[key_name].get_type())
                header_fixed_phrase_list = ["# .PCD v{} - Point Cloud Data file format\n",
                                            "VERSION {}\n", "FIELDS{}\n", "SIZE{}\n", "TYPE{}\n",
                                            "COUNT{}\n", "WIDTH {}\n", "HEIGHT {}\n", "VIEWPOINT {}\n",
                                            "POINTS {}\n", "DATA {}\n"]
                header_property_list = [PCD_VERSION, PCD_VERSION, header_fields, header_size, header_type,
                                        header_counts, width, height, "0 0 0 1 0 0 0", points, "ascii"]
                for header_fixed_phrase, header_property in zip(header_fixed_phrase_list, header_property_list):
                    pcd_file.write(header_fixed_phrase.format(header_property))

                # fields
                for point_data in point2_a:
                    set_point_str = ""
                    for key_name in offset_key_list:
                        point_val = offset_dict[key_name].get_field_data(point_data, big_endian)
                        set_point_str += "{} ".format(point_val)
                    else:
                        set_point_str += "\n"
                    pcd_file.write(set_point_str)
            result = True
        except IOError as e:
            logger.error(e)
        except Exception as e:
            logger.error(e)
        return result


def main():
    # ins = PcdCreator()
    # ins.run("2022-12-01T02:26:17.520+0900")
    pass


main()
