#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 point_cloud_field.py

 @author East Japan Accounting Center Co., Ltd

 (c)2023 The University of Aizu
 This software is released under the New BSD License.
"""

import logging
import struct

logger = logging.getLogger(__name__)


class DataType:
    class Value:
        def __init__(self, v: int, f: str, size: int, pf: str):
            self.__value = v
            self.__format = f
            self.__size = size
            self.__pcd_format = pf

        def get_value(self):
            return self.__value

        def get_format(self):
            return self.__format

        def get_size(self):
            return self.__size

        def get_pcd_format(self):
            return self.__pcd_format

    INT8 = Value(1, "b", 1, "I")
    UINT8 = Value(2, "B", 1, "U")
    INT16 = Value(3, "h", 2, "I")
    UINT16 = Value(4, "H", 2, "U")
    INT32 = Value(5, "i", 4, "I")
    UINT32 = Value(6, "I", 4, "U")
    FLOAT32 = Value(7, "f", 4, "F")
    FLOAT64 = Value(8, "d", 8, "F")
    __type_list_a = [INT8, UINT8, INT16, UINT16, INT32, UINT32, FLOAT32, FLOAT64]

    @classmethod
    def get_format(cls, tgt_type_id, endian):
        set_endian = "<"
        if endian:
            set_endian = ">"

        for type_id in cls.__type_list_a:
            if type_id.get_value() == tgt_type_id:
                return set_endian + type_id.get_format()
        else:
            return None

    @classmethod
    def get_size(cls, tgt_type_id):
        for type_id in cls.__type_list_a:
            if type_id.get_value() == tgt_type_id:
                return type_id.get_size()
        else:
            return 0

    @classmethod
    def get_pcd_format(cls, tgt_type_id):
        for type_id in cls.__type_list_a:
            if type_id.get_value() == tgt_type_id:
                return type_id.get_pcd_format()
        else:
            return ""


class PointCloudField:
    def __init__(self, field):
        self._name = field["name"]
        self._offset = field["offset"]
        self._datatype = field["datatype"]
        self._count = field["count"]

    def get_field_data(self, data_array, endian):
        s_pos = self._offset
        e_pos = s_pos + DataType.get_size(self._datatype)
        effective_range = data_array[s_pos:e_pos].tolist()
        set_format = DataType.get_format(self._datatype, endian)
        return struct.unpack(set_format, bytearray(effective_range))[0]

    def get_count(self):
        return self._count

    def get_size(self):
        return DataType.get_size(self._datatype)

    def get_type(self):
        return DataType.get_pcd_format(self._datatype)
